import json

from django.http import HttpResponse

from django_spire.ai.chat.messages import MessageGroup, Message, MessageType
from django_spire.ai.chat.models import Chat
from django_spire.ai.chat.tools import chat_workflow_process


def load_messages_render_view(request, chat_id):
    chat = (
        Chat.objects
        .by_user(request.user)
        .get(id=chat_id)
    )

    message_group = MessageGroup()

    for chat_message in chat.messages.all():
        message_group.add_message(
            chat_message.to_message(request)
        )

    return HttpResponse(message_group.render_to_html_string({'chat_id': chat.id}))


def request_message_render_view(request):
    body_data = json.loads(request.body)

    chat = (
        Chat.objects
        .by_user(request.user)
        .get(id=body_data['chat_id'])
    )

    if chat.is_empty:
        chat.name = body_data['message_body']
        chat.save()

    message_group = MessageGroup()

    user_message = Message(
        request=request,
        type=MessageType.REQUEST,
        sender='You',
        body=body_data['message_body']
    )

    message_group.add_message(
        user_message
    )

    chat.add_message(user_message)

    message_group.add_message(
        Message(
            request=request,
            type=MessageType.LOADING_RESPONSE,
            sender='Spire',
            body=body_data['message_body']
        )
    )

    return HttpResponse(message_group.render_to_html_string({'chat_id': chat.id}))


def response_message_render_view(request):
    body_data = json.loads(request.body)

    chat = (
        Chat.objects
        .by_user(request.user)
        .get(id=body_data['chat_id'])
    )

    response = chat_workflow_process(
        request,
        body_data['message_body'],
        message_history=chat.generate_message_history(),
    )

    spire_message = Message(
        request=request,
        type=MessageType.RESPONSE,
        sender='Spire',
        body=response['text'],
    )

    chat.add_message(spire_message)

    return HttpResponse(
        spire_message.render_to_html_string({'chat_id': chat.id})
    )



# Generated by Django 5.2 on 2025-04-11 21:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spire_ai', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aiinteraction',
            name='run_time_seconds',
            field=models.FloatField(default=0.0),
        ),
        migrations.AlterField(
            model_name='aiusage',
            name='run_time_seconds',
            field=models.FloatField(default=0.0),
        ),
    ]



from django.apps import AppConfig

from django_spire.tools import check_required_apps


class AiConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_spire.ai'
    label = 'spire_ai'
    REQUIRED_APPS = ('spire_core',)

    def ready(self) -> None:
        check_required_apps(self.label)



from __future__ import annotations


def generate_replacement_map(components: list[str]) -> dict[str, str]:
    app_name = components[-1]
    parent = components[-2] if len(components) > 1 else app_name
    module = '.'.join(components)

    return {
        'module': module,
        'spirepermission': parent.lower() + app_name.lower(),
        'SpireChildApp': app_name.capitalize(),
        'SpireChildApps': app_name.capitalize() + 's',
        'spirechildapp': app_name.lower(),
        'spirechildapps': app_name.lower() + 's',
        'SpireParentApp': parent.capitalize(),
        'SpireParentApps': parent.capitalize() + 's',
        'spireparentapp': parent.lower(),
        'spireparentapps': parent.lower() + 's'
    }



from django.urls import path

from django_spire.notification.app.views import page


app_name = 'spire_notification'

urlpatterns = [
    path('spire/notification/list/',
         view=page.app_notification_list_view,
         name='list'
    )
]



from django.urls import include, path

app_name = 'spire_app_notification'


urlpatterns = [
    path('ajax/', include('django_spire.notification.app.urls.ajax', namespace='ajax')),
    path('form/', include('django_spire.notification.app.urls.form', namespace='form')),
    path('page/', include('django_spire.notification.app.urls.page', namespace='page')),
    path('template/', include('django_spire.notification.app.urls.template', namespace='template')),
]



import unittest
from django.db import models
from django.utils.timezone import now

from django_spire.seeding import DjangoModelSeeder


class Product(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField()
    sku = models.CharField(max_length=20, unique=True)
    price = models.DecimalField(max_digits=6, decimal_places=2)
    in_stock = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(null=True, blank=True)

    class Meta:
        app_label = "tests"
        managed = False


class ProductSeeder(DjangoModelSeeder):
    model_class = Product
    fields = {
        "name": ('llm', 'Clothing store'),
        "description": ('static', 'A great product for testing.'),
        "price": ('faker', 'pydecimal', {'left_digits': 2, 'right_digits': 2, 'positive': True}),
        "in_stock": True,
        "created_at": ("callable", now),
        "updated_at": 'exclude',
    }


class TestProductSeeder(unittest.TestCase):

    def test_assign_defaults(self):
        self.assertIn('sku', ProductSeeder.resolved_fields().keys())

    def test_seeds_three_objects(self):
        result = ProductSeeder.seed(count=3)
        self.assertEqual(len(result), 3)
        for obj in result:
            self.assertIsInstance(obj, Product)

    def test_raises_for_invalid_field(self):
        with self.assertRaises(Exception) as context:
            ProductSeeder.seed(count=1, fields={"not_a_field": ("static", "value")})
        self.assertIn("not_a_field", str(context.exception))

    def test_field_override_applies(self):
        override_time = now()
        result = ProductSeeder.seed(count=1, fields={"updated_at": ("static", override_time)})
        self.assertEqual(result[0].updated_at, override_time)



from dandy.llm import BaseLlmBot, LlmConfigOptions


from django_spire.seeding.intelligence.intel import SourceIntel


from django_spire.seeding.intelligence.prompts.generate_django_model_seeder_prompts import \
    generate_django_model_seeder_user_prompt, generate_django_model_seeder_system_prompt


class SeederGeneratorBot(BaseLlmBot):

    config = 'DEEP_SEEK'

    config_options = LlmConfigOptions(
        temperature=0.3,
        randomize_seed=True
    )

    @classmethod
    def process(
            cls,
            model_import: str,
            model_description: str
    ):
        return cls.process_prompt_to_intel(
            prompt=generate_django_model_seeder_user_prompt(
                model_import,
                model_description,
            ),
            intel_class=SourceIntel,
            postfix_system_prompt=generate_django_model_seeder_system_prompt()
        )



from django.apps import AppConfig

from django_spire.tools import check_required_apps


class NotificationConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_spire.notification'
    label = 'spire_notification'
    REQUIRED_APPS = ('spire_core', 'spire_history', 'spire_history_viewed')

    def ready(self) -> None:
        check_required_apps(self.label)



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django_spire.authentication.mfa.models import MfaCode

if TYPE_CHECKING:
    from django.contrib.auth.models import User


def get_or_generate_user_mfa_code(user: User) -> MfaCode:
    mfa_code = MfaCode.objects.valid_code(user)

    if mfa_code:
        return mfa_code

    return MfaCode.generate_code(user)



import unittest
from copy import copy
from django_spire.seeding.field.cleaners import normalize_seeder_fields


FIELDS = {
    "faker_tuple": ("faker", "name"),
    "llm_type": "llm",
    "faker_type": "faker",
    "static_type": "static",
    "callable_type": "callable",
    "custom_type": "custom",
    "static_bool": True,
    "static_str": "approved",
    "static_int": 10,
    "callable_func": lambda: "now",
    "exclude_str": "exclude",
    "exclude_tuple": ("exclude",)
}


class TestNormalizeSeederFields(unittest.TestCase):

    def test_normalizes_faker_tuple(self):
        fields = {"faker_tuple": FIELDS["faker_tuple"]}
        normalized = normalize_seeder_fields(fields)
        self.assertEqual(normalized["faker_tuple"], ("faker", "name"))

    def test_excludes_fields(self):
        fields = copy(FIELDS)
        normalized = normalize_seeder_fields(fields)
        self.assertNotIn("exclude_str", normalized)
        self.assertNotIn("exclude_tuple", normalized)
        self.assertIn("static_str", normalized)

    def test_normalizes_single_value_strings_to_tuples(self):
        fields = {
            k: FIELDS[k] for k in (
                "llm_type", "faker_type", "static_type", "callable_type", "custom_type"
            )
        }
        normalized = normalize_seeder_fields(fields)
        for key, value in fields.items():
            self.assertEqual(normalized[key], (value,))

    def test_normalizes_callable(self):
        fields = {"callable_func": FIELDS["callable_func"]}
        normalized = normalize_seeder_fields(fields)
        self.assertEqual(normalized["callable_func"][0], "callable")
        self.assertTrue(callable(normalized["callable_func"][1]))

    def test_normalizes_static_values(self):
        fields = {
            "static_bool": FIELDS["static_bool"],
            "static_str": FIELDS["static_str"],
            "static_int": FIELDS["static_int"]
        }
        normalized = normalize_seeder_fields(fields)
        self.assertEqual(normalized["static_bool"], ("static", True))
        self.assertEqual(normalized["static_str"], ("static", "approved"))
        self.assertEqual(normalized["static_int"], ("static", 10))



# Generated by Django 5.2 on 2025-04-14 15:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spire_ai', '0002_alter_aiinteraction_run_time_seconds_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='aiusage',
            name='was_successful',
            field=models.BooleanField(default=True, verbose_name='Success'),
        ),
        migrations.AlterField(
            model_name='aiinteraction',
            name='event_count',
            field=models.IntegerField(default=0, verbose_name='Events'),
        ),
        migrations.AlterField(
            model_name='aiinteraction',
            name='run_time_seconds',
            field=models.FloatField(default=0.0, verbose_name='Run Time'),
        ),
        migrations.AlterField(
            model_name='aiinteraction',
            name='token_usage',
            field=models.IntegerField(default=0, verbose_name='Tokens'),
        ),
        migrations.AlterField(
            model_name='aiinteraction',
            name='was_successful',
            field=models.BooleanField(default=True, verbose_name='Success'),
        ),
        migrations.AlterField(
            model_name='aiusage',
            name='event_count',
            field=models.IntegerField(default=0, verbose_name='Events'),
        ),
        migrations.AlterField(
            model_name='aiusage',
            name='run_time_seconds',
            field=models.FloatField(default=0.0, verbose_name='Run Time'),
        ),
        migrations.AlterField(
            model_name='aiusage',
            name='token_usage',
            field=models.IntegerField(default=0, verbose_name='Tokens'),
        ),
    ]



from django.db import models

from django_spire.notification.models import Notification

class SmsNotification(models.Model):
    notification = models.OneToOneField(Notification, editable=False, on_delete=models.CASCADE)
    subject = models.CharField(max_length=128)
    phone_number = models.CharField(max_length=11, blank=True)

    def __str__(self):
        return f'{self.notification.title} - {self.number}'

    class Meta:
        db_table = 'spire_notification_sms'
        verbose_name = 'SMS Notification'
        verbose_name_plural = 'SMS Notifications'


from __future__ import annotations
from typing_extensions import TYPE_CHECKING

from django.template.response import TemplateResponse

from django_spire.notification.app.models import AppNotification

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest

def notification_dropdown_template_view(request: WSGIRequest) -> TemplateResponse:
    app_notification_list = (
        AppNotification.objects.active()
        .annotate_is_viewed_by_user(request.user)
        .order_by('-created_datetime')
        .select_related('notification')
        .distinct()
    )

    return TemplateResponse(
        request,
        context={
            'app_notification_list': app_notification_list,
        },
        template='spire/notification/app/dropdown/notification_dropdown_content.html'
    )



# Generated by Django 5.1.3 on 2024-11-08 22:57

import django.contrib.auth.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='PortalGroup',
            fields=[
            ],
            options={
                'verbose_name': 'Group',
                'verbose_name_plural': 'Groups',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.group', models.Model),
            managers=[
                ('objects', django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.CreateModel(
            name='PortalUser',
            fields=[
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.user', models.Model),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
    ]



from __future__ import annotations

from django.db import models
from django.utils.timezone import now
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey

from django_spire.history.mixins import HistoryModelMixin
from django_spire.notification.choices import (
    NotificationSenderMap,
    NotificationTypeChoices
)

class Notification(HistoryModelMixin):
    type = models.CharField(max_length=32, default=NotificationTypeChoices.EMAIL, choices=NotificationTypeChoices.choices)
    title = models.CharField(max_length=124)
    body = models.TextField(default='')
    processed_datetime = models.DateTimeField(blank=True, null=True)
    url = models.CharField(max_length=255, default='')
    is_processed = models.BooleanField(default=False)

    content_object = GenericForeignKey('content_type', 'object_id')
    content_type = models.ForeignKey(ContentType, related_name='spire_notification', on_delete=models.CASCADE, editable=False)
    object_id = models.PositiveIntegerField()

    def mark_processed(self) -> None:
        self.processed = True
        self.processed_datetime = now()
        self.save()

    def send(self) -> None:
        sender_class = NotificationSenderMap(self.type).value
        sender = sender_class(self)
        sender.send()

    class Meta:
        db_table = 'spire_notification'
        verbose_name = 'Notification'
        verbose_name_plural = 'Notifications'



from django.contrib import admin

from django_spire.notification.email.models import EmailNotification

# Register your models here.
@admin.register(EmailNotification)
class EmailNotificationAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'notification', 'subject', 'email'
    )



import json

from dandy.llm import MessageHistory
from dandy.llm.service.request.message import RoleLiteralStr
from django.contrib.auth.models import User
from django.db import models
from django.utils.timezone import now

from django_spire.ai.chat.messages import Message, MessageType
from django_spire.ai.chat.query_sets import ChatQuerySet
from django_spire.history.mixins import HistoryModelMixin


class Chat(HistoryModelMixin):
    user = models.ForeignKey(
        User,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='chats',
        related_query_name='chat'
    )

    name = models.CharField(max_length=128)
    last_message_datetime = models.DateTimeField(default=now, editable=False)
    has_unread_messages = models.BooleanField(default=False)

    objects = ChatQuerySet.as_manager()

    def __str__(self):
        if len(self.name) < 48:
            return self.name

        return self.name[:48] + '...'

    def add_message(self, message: Message) -> None:
        ChatMessage.objects.create(
            chat=self,
            content=message.to_json(),
            is_processed=True,
            is_viewed=True
        )
        self.last_message_datetime = now()
        self.save()

    def generate_message_history(
            self,
            message_count: int = 20,
            exclude_last_message: bool = True
    ) -> MessageHistory:
        message_history = MessageHistory()

        messages = self.messages.order_by('-created_datetime')[:message_count]

        if exclude_last_message:
            messages = messages[1:]

        messages = list(reversed(messages))

        for message in messages:
            print(f'{message.role}: {message.body}')
            message_history.add_message(
                role=message.role,
                content=message.body
            )

        return message_history

    @property
    def is_empty(self) -> bool:
        return self.messages.count() == 0

    @property
    def name_shortened(self) -> str:
        if len(self.name) > 24:
            return self.name[:24] + '...'

        return self.name

    class Meta:
        db_table = 'spire_ai_chat'
        verbose_name = 'Chat'
        verbose_name_plural = 'Chats'
        ordering = ('-last_message_datetime', 'name')


class ChatMessage(HistoryModelMixin):
    chat = models.ForeignKey(
        Chat,
        on_delete=models.CASCADE,
        related_name='messages',
        related_query_name='message'
    )

    content = models.JSONField()
    is_processed = models.BooleanField(default=False)
    is_viewed = models.BooleanField(default=False)

    def __str__(self):
        body = json.loads(self.content)['body']

        if len(body) < 64:
            return body

        return body[:64] + '...'

    @property
    def body(self):
        return json.loads(self.content)['body']

    @property
    def role(self) -> RoleLiteralStr:
        message_type = json.loads(self.content)['type']
        if message_type == 'request':
            return 'user'
        elif message_type == 'response':
            return 'assistant'
        else:
            return 'system'

    def to_message(self, request) -> Message:
        content = json.loads(self.content)

        return Message(
            request=request,
            type=MessageType(content['type']),
            sender=content['sender'],
            body=content['body']
        )

    class Meta:
        db_table = 'spire_ai_chat_message'
        verbose_name = 'Chat Message'
        verbose_name_plural = 'Chat Messages'


# Generated by Django 5.2 on 2025-04-11 21:21

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Chat',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('name', models.CharField(max_length=128)),
                ('last_message_datetime', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('has_unread_messages', models.BooleanField(default=False)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='chats', related_query_name='chat', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Chat',
                'verbose_name_plural': 'Chats',
                'db_table': 'spire_ai_chat',
            },
        ),
        migrations.CreateModel(
            name='ChatMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('content', models.JSONField()),
                ('is_processed', models.BooleanField(default=False)),
                ('is_viewed', models.BooleanField(default=False)),
                ('chat', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', related_query_name='message', to='spire_ai_chat.chat')),
            ],
            options={
                'verbose_name': 'Chat Message',
                'verbose_name_plural': 'Chat Messages',
                'db_table': 'spire_ai_chat_message',
            },
        ),
    ]



from __future__ import annotations

import random

from dateutil import relativedelta

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.db import models
from django.urls import reverse
from django.utils.timezone import localtime

from django_spire.notification.email.sender import SendGridEmailHelper
from django_spire.authentication.mfa.querysets import MfaCodeQuerySet


class MfaCode(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_query_name='mfa_code', related_name='mfa_codes')
    code = models.CharField(max_length=6, editable=False, unique=True)
    expiration_datetime = models.DateTimeField(editable=False)

    objects = MfaCodeQuerySet.as_manager()

    def __str__(self):
        return f'{self.expiration_datetime} - {self.code}'

    def is_valid(self) -> bool:
        return self.expiration_datetime > localtime()

    def set_expired(self) -> None:
        self.expiration_datetime = localtime()
        self.save()

    def send_notification(self):
        user = self.user

        context_data = {
            'title': 'Authentication Code',
            'body': f"Here is your multifactor authentication code: </br> <h3>{self.code}</h3>",
            'name': user.first_name,
            'button_url': f'{Site.objects.get_current()}/{reverse("authentication:redirect:login_redirect")[1:]}'
        }

        # Directly connecting to SendGrid to send MFA email faster
        SendGridEmailHelper(
            to=[user.email],
            template_id='d-9ae6be6be95d4f79b0daad9055f03cc9',
            template_data=context_data,
            fail_silently=False
        ).send()

    @classmethod
    def generate_code(cls, user):
        return MfaCode.objects.create(
            user=user,
            code=int(''.join([str(random.randint(0, 9)) for _ in range(6)])),
            expiration_datetime=localtime() + relativedelta.relativedelta(minutes=5)
        )

    class Meta:
        db_table = 'spire_authentication_mfa_code'



# Generated by Django 5.2 on 2025-04-14 15:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spire_ai_chat', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='chat',
            options={'ordering': ('-last_message_datetime', 'name'), 'verbose_name': 'Chat', 'verbose_name_plural': 'Chats'},
        ),
    ]



from django.urls import path

from django_spire.ai.chat.views import template_views


app_name = 'template'

urlpatterns = [
    path('load/', template_views.load_template_view, name='load'),
    path('recent/', template_views.recent_template_view, name='recent'),
    path('search/', template_views.search_template_view, name='search'),
]



from __future__ import annotations
from typing import TYPE_CHECKING
from django.db.models import QuerySet, Value, When, Case, BooleanField

if TYPE_CHECKING:
    from django.contrib.auth.models import User

class AppNotificationQuerySet(QuerySet):
    def active(self) -> QuerySet:
        return self.filter(is_active=True, is_deleted=False)

    def by_user(self, user: User) -> QuerySet:
        return self.filter(user=user)

    def exclude_viewed_by_user(self, user: User) -> QuerySet:
        return self.by_user(user=user).exclude(views__user=user)

    def annotate_is_viewed_by_user(self, user: User) -> QuerySet:
        return self.by_user(user=user).annotate(viewed=Case(
            When(views__user=user, then=Value(True)),
            default=Value(False),
            output_field=BooleanField()
        ))



# Generated by Django 5.2 on 2025-04-11 21:12

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('type', models.CharField(choices=[('app', 'App'), ('email', 'Email'), ('push', 'Push'), ('sms', 'Sms')], default='email', max_length=32)),
                ('title', models.CharField(max_length=124)),
                ('body', models.TextField(default='')),
                ('processed_datetime', models.DateTimeField(blank=True, null=True)),
                ('url', models.CharField(default='', max_length=255)),
                ('is_processed', models.BooleanField(default=False)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='spire_notification', to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
                'db_table': 'spire_notification',
            },
        ),
    ]



from __future__ import annotations

from django import template


register = template.Library()


@register.simple_tag
def check_permission(user, app_label: str, model_name: str, action: str):
    return user.has_perm(f'{app_label}.{action}_{model_name}')



from django.urls import path

from django_spire.home import views


app_name = 'home'

urlpatterns = [
    path('', views.home_view, name='home'),

    # Path has to match Maintenance Mode Middleware
    path(
        'maintenance/mode',
        views.maintenance_mode_view,
        name='maintenance_mode'),
]



from __future__ import annotations

from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Row, Column, HTML


class MFAForm(forms.Form):
    def __init__(self, generated_mfa_code, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper(self)
        self.helper.include_media = False
        self.generated_mfa_code = generated_mfa_code
        self.fields['mfa_code'] = forms.CharField(max_length=6, label='Authentication Code')

        self.helper.layout = Layout(
            Row(
                Column('mfa_code', css_class='form-group col-12'),
            ),
            HTML("{% include 'spire/form/form_submit_button.html' %}")
        )

    def clean(self) -> None:
        cleaned_data = super().clean()

        mfa_code = cleaned_data.get('mfa_code')

        if mfa_code != self.generated_mfa_code.code:
            self.add_error('mfa_code', 'Invalid MFA Code.')



from django.db import models

from django_spire.history.mixins import HistoryModelMixin


class PushNotification(HistoryModelMixin):

    class Meta:
        db_table = 'spire_notification_push'
        verbose_name = 'Push Notification'
        verbose_name_plural = 'Push Notifications'


from django.urls import path

from django_spire.ai.chat.views import render_views


app_name = 'render'

urlpatterns = [
    path('<int:chat_id>/load/messages/', render_views.load_messages_render_view, name='load_messages'),
    path('request/message/', render_views.request_message_render_view, name='request_message'),
    path('response/message/', render_views.response_message_render_view, name='response_message'),
]



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.template.response import TemplateResponse

from django_spire.core.redirect.generic_redirect import reverse_generic_relation

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def search_view(request: WSGIRequest) -> TemplateResponse:
    context_data = {}

    if 'query' in request.GET:
        search_query = request.GET['query']
        search_item_list = []
        model_info_dict = [
            {
                'app_label': '',
                'model_name': '',
                'query_set': []
            }
        ]

        for model in model_info_dict:
            for item in model['query_set']:
                item_name = item

                if 'model_identifier' in model:
                    model_name = model['model_identifier']
                else:
                    model_name = model['model_name']

                search_item_list.append({
                    'name': item_name,
                    'model_name': model_name,
                    'url': reverse_generic_relation(item)
                })

        context_data['search_query'] = search_query
        context_data['search_item_list'] = search_item_list

    return TemplateResponse(request, 'spire/page/full_search_page.html', context_data)



from django.apps import apps

def check_required_apps(app_label: str) -> None:
    app_config = apps.get_app_config(app_label)
    for required_app_name in app_config.REQUIRED_APPS:
        if required_app_name not in list(apps.app_configs.keys()):
            raise Exception(f"{app_label} requires {required_app_name} is be in the 'INSTALLED_APPS' list before {app_label} in the django settings module.")



import json
from dataclasses import dataclass, field, asdict
from enum import Enum
from typing import List

from django.http import HttpRequest
from django.template.loader import render_to_string


class MessageType(Enum):
    REQUEST = 'request'
    RESPONSE = 'response'
    LOADING_RESPONSE = 'loading_response'


@dataclass
class Message:
    request: HttpRequest
    type: MessageType
    sender: str
    body: str

    @property
    def context_data(self) -> dict:
        return {
            'sender': self.sender,
            'body': self.body,
            'request': self.request
        }

    def _render_template_to_html_string(self, template_name: str, context_data: dict = None) -> str:
        return render_to_string(
            template_name=template_name,
            context={**self.context_data, **(context_data or {})},
        )

    def render_to_html_string(self, context_data: dict = None) -> str:
        if self.type == MessageType.REQUEST:
            return self._render_template_to_html_string('spire/ai/chat/message/request_message.html', context_data)
        elif self.type == MessageType.RESPONSE:
            return self._render_template_to_html_string('spire/ai/chat/message/response_message.html', context_data)
        elif self.type == MessageType.LOADING_RESPONSE:
            return self._render_template_to_html_string('spire/ai/chat/message/loading_response_message.html', context_data)

    def to_dict(self) -> dict:
        return {
            'type': self.type.value,
            'sender': self.sender,
            'body': self.body
        }

    def to_json(self) -> str:
        return json.dumps(self.to_dict())


@dataclass
class MessageGroup:
    messages: List[Message] = field(default_factory=list)

    def add_message(self, message: Message) -> None:
        self.messages.append(message)

    def render_to_html_string(self, context_data: dict = None) -> str:
        html_string = ''
        for message in self.messages:
            html_string += message.render_to_html_string(context_data)

        return html_string



from django.urls import path

from django_spire.ai.chat.views import json_views


app_name = 'json'

urlpatterns = [
    path('workflow_process', json_views.workflow_process_json_view, name='workflow_process'),
]



from django.urls import path

from django_spire.ai import views


app_name = 'speech_to_text'

urlpatterns = [
    path(
        'transcribe/',
        views.transcribe_audio,
        name='transcribe'
    )
]



from __future__ import annotations

from dataclasses import dataclass, field


@dataclass
class Option:
    key: str
    value: str | bool | int


@dataclass
class OptionSection:
    name: str
    options: list[Option] = field(default_factory=list)

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()

    def __contains__(self, option_key: str) -> bool:
        return any(option.key == option_key for option in self.options)

    def __getitem__(self, option_key: str) -> Option:
        for option in self.options:
            if option.key.lower() == option_key.lower():
                return option

        message = f'Option "{option_key}" not found'
        raise KeyError(message)

    def to_dict(self):
        section_dict = {}

        for option in self.options:
            section_dict[option.key.lower()] = option.value

        return section_dict


@dataclass
class Options:
    sections: list[OptionSection] = field(default=list)

    def __eq__(self, other):
        # Compares the section names and option keys to sync options.
        section_names = sorted([section.name.lower() for section in self.sections])
        other_section_names = sorted([section.name.lower() for section in other.sections])

        if section_names != other_section_names:
            return False

        for section in self.sections:
            option_keys = sorted([option.key.lower() for option in section.options])
            other_option_keys = sorted([option.key.lower() for option in other[section.name].options])

            if option_keys != other_option_keys:
                return False

        return True

    def __contains__(self, section_name: str) -> bool:
        return any(section.name == section_name for section in self.sections)

    def __getitem__(self, section_name: str) -> OptionSection:
        for section in self.sections:
            if section.name.lower() == section_name.lower():
                return section

        message = f'Section "{section_name}" not found'
        raise KeyError(message)

    @classmethod
    def load_dict(cls, options_dict: dict):
        sections = []

        for section_name, section_options in options_dict.items():
            options = []

            for option_key, option_value in section_options.items():
                options.append(Option(key=option_key, value=option_value))

            sections.append(OptionSection(name=section_name, options=options))

        return cls(sections=sections)

    def get_setting(self, section_name: str, option_key: str):
        return self[section_name][option_key].value

    def sync_options(self, default_options: 'Options'):
        new_sections = []
        for section in default_options.sections:
            new_section = OptionSection(name=section.name)

            for option in section.options:
                key = option.key
                if section.name in self and key in self[section.name]:
                    value = self[section.name][key].value
                else:
                    value = option.value

                new_section.options.append(Option(key=key, value=value))

            new_sections.append(new_section)

        self.sections = new_sections

    def update_setting(self, section_name: str, option_key: str, value: str | bool):
        self[section_name][option_key].value = value

    def to_dict(self):
        options_dict = {}

        for section in self.sections:
            options_dict[section.name.lower()] = section.to_dict()

        return options_dict



from django.db import models

from django_spire.notification.models import Notification

class EmailNotification(models.Model):
    notification = models.OneToOneField(Notification, editable=False, on_delete=models.CASCADE)
    subject = models.CharField(max_length=128)
    email = models.EmailField()

    def __str__(self):
        return f'{self.notification.title} - {self.email}'

    class Meta:
        db_table = 'spire_notification_email'
        verbose_name = 'Email Notification'
        verbose_name_plural = 'Email Notifications'


import json

from django.template.response import TemplateResponse
from django.utils.timezone import now

from django_spire.ai.chat.models import Chat


def load_template_view(request):
    body_data = json.loads(request.body)

    chat_id = body_data['chat_id']

    if chat_id == 0:
        chat = Chat.objects.get_empty_or_create(
            user=request.user
        )
        chat.name = 'New Chat'
        chat.last_message_datetime = now()
        chat.save()

    else:
        chat = Chat.objects.get(
            id=chat_id,
            user=request.user,
        )

    return TemplateResponse(
        request,
        'spire/ai/chat/widget/dialog_widget.html',
        {
            'chat': chat,
            'height_correction': body_data['height_correction']
        }
    )

def recent_template_view(request):
    chats = (
        Chat.objects
        .by_user(request.user)
        .order_by('-last_message_datetime')
    )[:20]

    return TemplateResponse(
        request,
        'spire/ai/chat/widget/recent_chat_list_widget.html',
        {
            'recent_chats': [
                {
                    'name': chat.name_shortened,
                    'id': chat.id,
                } for chat in chats
            ]
        }
    )

def search_template_view(request):
    body_data = json.loads(request.body)
    chats = Chat.objects.by_user(request.user).search(body_data['query'])

    return TemplateResponse(
        request,
        'spire/ai/chat/widget/search_chat_results_widget.html',
        {
            'search_results': [
                {
                    'name': chat.name,
                    'id': chat.id,
                } for chat in chats
            ]
        }
    )


from pathlib import Path

from django.core.management.base import BaseCommand, CommandError

from dandy.conf import settings
from dandy.recorder import recorder_to_html_file

from django_spire.seeding.intelligence.bots.seeder_generator_bot import SeederGeneratorBot


_SEEDING_OUTPUT_PATH = Path(settings.BASE_PATH, '.seeding_generator_output')


class Command(BaseCommand):
    help = 'Generate a Seeder'

    def add_arguments(self, parser):
        parser.add_argument(
            'model_import',
            type=str,
            help='The import path to the Django model class',
        )

        parser.add_argument(
            'model_description',
            type=str,
            help='A small descript of the model',
            nargs='+',
        )

    @recorder_to_html_file('seeding_generator')
    def handle(self, *args, **kwargs):
        if not kwargs['model_import'] or not kwargs['model_description']:
            raise CommandError('You must provide a model import path and a model description')

        model_import = kwargs['model_import']
        model_description = ' '.join(kwargs['model_description'])

        source_intel = SeederGeneratorBot.process(model_import, model_description)

        Path(_SEEDING_OUTPUT_PATH).mkdir(parents=True, exist_ok=True)

        with open(Path(_SEEDING_OUTPUT_PATH, source_intel.file_name), 'w') as f:
            f.write(source_intel.python_source_code)

        self.stdout.write(f'Done ... saved to "{Path(_SEEDING_OUTPUT_PATH, source_intel.file_name)}"')



from __future__ import annotations
from typing import TYPE_CHECKING

from django.contrib.contenttypes.fields import GenericRelation
from django.db import models

from django_spire.history.viewed.models import Viewed

if TYPE_CHECKING:
    from django.contrib.auth.models import User

class ViewedModelMixin(models.Model):
    views = GenericRelation(
        Viewed,
        related_query_name='views',
        editable=False
    )

    def add_view(self, user: User) -> None:
        self.views.create(user=user)

    def is_viewed(self, user: User) -> None:
        return self.views.filter(user=user).exists()

    class Meta:
        abstract = True



import json

from django.http import JsonResponse

from django_spire.ai.chat.tools import chat_workflow_process


def workflow_process_json_view(request) -> JsonResponse:
    body = json.loads(request.body.decode('utf-8'))

    response = chat_workflow_process(
        request,
        body
    )

    return JsonResponse({
        'response': response
    })


from __future__ import annotations
from abc import ABC, abstractmethod
from typing_extensions import Any, TYPE_CHECKING

from django.contrib.contenttypes.models import ContentType

from django_spire.notification.models import Notification
from django_spire.notification.choices import NotificationTypeChoices
from django_spire.notification.app.models import AppNotification
from django_spire.notification.email.models import EmailNotification
from django_spire.notification.sms.models import SmsNotification
from django_spire.notification.push.models import PushNotification

if TYPE_CHECKING:
    from django.db import models
    from django.contrib.auth.models import User


class BaseNotificationFactory(ABC):
    """
    Abstract base class for notification factories.

    This class serves as a blueprint for creating different types of notifications.
    Subclasses should implement specific methods to create various kinds of notifications.

    Attributes:
        model_object (Any[models.Model]): The model object associated with the notification.
        title (str): The title or main text of the notification.
        body (Optional[str]): Additional content or details of the notification. Defaults to None.
        url (Optional[str]): URL associated with the notification, if any. Defaults to None.

    """

    def __init__(self, model_object: Any[models.Model], title: str, body: str | None = None, url: str | None = None):
        self.model_object = model_object
        self.title = title
        self.body = body
        self.url = url


    def create_notification(self, notification_type: NotificationTypeChoices) -> Notification:
        """
        Method for creating base `Notification` objects.

        :param notification_type (str): The type of the notification.

        :return Notification: A new instance of the specified notification.
        """
        return Notification.objects.create(
                content_type=ContentType.objects.get_for_model(self.model_object),
                object_id=self.model_object.id,
                title=self.title,
                body=self.body,
                url=self.url,
                type=notification_type,
            )

    @abstractmethod
    def create_email_notification(self, email: str) -> EmailNotification:
        """
        Abstract method for sending notification by Email
        :param: email(str): The recipient's email address for the notification.
        :return EmailNotification: A new instance of an email notification.
        """

    @abstractmethod
    def create_app_notification(self, model_object: Any[models.Model], user: Any[User], url: str|None = None) -> AppNotification:
        """
        Abstract method for creating in-app notification linked to a model object
        :param model_object: The model object associated with the notification.
        :param user: The user who will receive the notification.
        :param url (Optional[str]): URL related to the notification, if required override to url specified in factory.
        """

    # TODO: Implementation & Testing
    # @abstractmethod
    # def create_push_notification(self) -> PushNotification:
    #     """Abstract method for sending push notifications in a PWA"""
    #     pass

    # TODO: Implementation & Testing
    # @abstractmethod
    # def create_sms_notification(self, phone_number: str) -> SmsNotification:
    #     """Abstract method for sending SMS notifications"""
    #     pass


class NotificationFactory(BaseNotificationFactory):
    """
    Concrete implementation for creating different types of notifications.

    This class provides methods to create specific types of notifications, such as email and app notifications.
    """

    def create_email_notification(self, email: str) -> EmailNotification:
        """
        Creates an email notification.
        :param email (str): The recipient's email address for the notification.
        :return EmailNotification: A new instance of an email notification.
        """

        return EmailNotification.objects.create(
            notification=self.create_notification(NotificationTypeChoices.EMAIL),
            subject=self.title,
            email=email
        )

    def create_app_notification(
        self,
        user: Any[User],
    ) -> AppNotification:
        """
        Creates an app notification.

        :param user (User): The user who will receive the notification.
        :param model_object (models.Model): The model object associated with the notification.

        :return AppNotification: A new instance of an app notification.
        """
        return AppNotification.objects.create(
            notification=self.create_notification(NotificationTypeChoices.APP),
            user=user,
        )



# Generated by Django 5.2 on 2025-04-11 21:21

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AiUsage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('event_count', models.IntegerField(default=0)),
                ('token_usage', models.IntegerField(default=0)),
                ('run_time_seconds', models.IntegerField(default=0)),
                ('recorded_date', models.DateField(default=django.utils.timezone.now)),
            ],
            options={
                'verbose_name': 'AI Usage',
                'verbose_name_plural': 'AI Usage',
                'db_table': 'spire_ai_usage',
            },
        ),
        migrations.CreateModel(
            name='AiInteraction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('event_count', models.IntegerField(default=0)),
                ('token_usage', models.IntegerField(default=0)),
                ('run_time_seconds', models.IntegerField(default=0)),
                ('actor', models.CharField(max_length=100)),
                ('user_email', models.EmailField(blank=True, max_length=254, null=True)),
                ('user_first_name', models.CharField(blank=True, max_length=100, null=True)),
                ('user_last_name', models.CharField(blank=True, max_length=100, null=True)),
                ('module_name', models.TextField()),
                ('callable_name', models.TextField()),
                ('was_successful', models.BooleanField(default=True)),
                ('exception', models.TextField(blank=True, null=True)),
                ('stack_trace', models.TextField(blank=True, null=True)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('interaction', models.JSONField(blank=True, null=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ai_interactions', related_query_name='ai_interaction', to=settings.AUTH_USER_MODEL)),
                ('ai_usage', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='interactions', related_query_name='interaction', to='spire_ai.aiusage')),
            ],
            options={
                'verbose_name': 'AI Interaction',
                'verbose_name_plural': 'AI Interactions',
                'db_table': 'spire_ai_interaction',
            },
        ),
    ]



from importlib import import_module

from dandy.llm import MessageHistory
from dandy.workflow import BaseWorkflow
from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest
from pydantic import BaseModel

from django_spire.ai.decorators import log_ai_interaction_from_recorder
from django_spire.consts import AI_CHAT_WORKFLOW_CLASS_NAME


def chat_workflow_process(
        request: WSGIRequest,
        user_input: str | None = None,
        message_history: MessageHistory | None = None,
) -> BaseModel:
    if user_input is None:
        raise ValueError('user_input is required')

    chat_workflow_class = getattr(settings, AI_CHAT_WORKFLOW_CLASS_NAME)

    module_name = '.'.join(chat_workflow_class.split('.')[:-1])
    object_name = chat_workflow_class.split('.')[-1]

    try:
        workflow_module = import_module(module_name)
    except ImportError:
        raise ImportError(f'Could not import workflow module: {module_name}')

    ChatWorkFlow: BaseWorkflow = getattr(workflow_module, object_name)

    @log_ai_interaction_from_recorder(request.user)
    def run_workflow_process():
        return ChatWorkFlow.process(
            request=request,
            user_input=user_input,
            message_history=message_history,
        )

    return run_workflow_process()



# from dandy.llm import Prompt
# from django.test import TestCase
#
# from django_spire.seeding.processor import SeedingProcessor
# from django_spire.seeding.models import PersonSeedingModel
#
#
# class SeedingTestCase(TestCase):
#     def setUp(self):
#         self.person_seed_processor = SeedingProcessor(
#             model_class=PersonSeedingModel,
#             seeding_prompt=(
#                 Prompt()
#                 .text('Create a bunch of unique people to be used for seeding.')
#             ),
#             count=5,
#             exclude_fields=['phone_number'],
#         )
#
#     def tearDown(self):
#         pass
#
#     def test_seeding_helper_intel_class(self):
#         person_intel_class = self.person_seed_processor.build_intel_class()
#



from __future__ import annotations

from enum import Enum

from django.db import models

from django_spire.notification.email.sender import EmailNotificationSender


class NotificationTypeChoices(models.TextChoices):
    APP = 'app'
    EMAIL = 'email'
    PUSH = 'push'
    SMS = 'sms'

    def send(self) -> None:
        sender_class = NotificationSenderMap(self.value).value
        sender = sender_class(self)
        sender.send()


class NotificationSenderMap(Enum):
    EMAIL = EmailNotificationSender
    # SMS = EmailNotificationSender
    # PUSH = EmailNotificationSender
